------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Script is loading.
env.info("Loading MOOSE Rescue Helo-Carrier Formation Script v1.1 by FF")

-- v1.1: Made AI_FORMATION object global variable since garbage collector removed it.
--       Used new RespawnAtAirbase() function. Requires MOOSE version >= 2.4 

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Set parameters.

-- Carrier UNIT name as set in the ME. 
local CarrierUnitName="USS Stennis"
local HangarUnitName="Normandy"

-- Helo GROUP name as set in the ME.
local HeloGroupName="Rescue Helo"

-- Offset [meters] in the direction of travelling. Positive values are in front of Mother.
local OffsetX=300
-- Offset [meters] perpendicular to travelling. Positive = Starboard (right of Mother), negative = Port (left of Mother).
local OffsetZ=200
-- Offset altitude. Should (obviously) always be positve.
local OffsetY=80

-- If true, Carrier will randomly select waypoints from the ones defined in the ME.
local RandomRoute=false

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Scripting Part - no need to change anything below unless you know what you are doing.

-- Report mission time every 30 sec.
local ReportTime=false

-- Report fuel of helo group every 60 sec.
local ReportFuel=false

-- Display mission time.
if ReportTime then
  local Scheduler_Mission_Time = SCHEDULER:New(nil, UTILS.DisplayMissionTime, {}, 0, 30)
end

-- Debug info.
env.info(string.format("Carrier Unit Name = %s", CarrierUnitName))
env.info(string.format("Helo Group Name   = %s", HeloGroupName))
env.info(string.format("Offset X (front)  = %.1f m", OffsetX))
env.info(string.format("Offset Z (port)   = %.1f m", OffsetZ))
env.info(string.format("Offset Y (alt)    = %.1f m", OffsetY))
env.info(string.format("Spawn in air      = %s", tostring(SpawnInAir)))
env.info(string.format("Random Route      = %s", tostring(RandomRoute)))

-- Set takeoff type.
local Takeoff=SPAWN.Takeoff.Hot  -- Adjust this to SPAWN.Takeoff.Cold for a cold start, i.e. with engines off. 

-- Carrier "USS Stennis" = Mother. "USS Stennis" is the name of the unit of the carrier given to the carrier in the ME. 
local Mother=UNIT:FindByName(CarrierUnitName)



-- Spawn helo.
local SpawnHelo=SPAWN:New(HeloGroupName)
local HeloSpawn=SpawnHelo:SpawnAtAirbase(AIRBASE:FindByName(HangarUnitName), Takeoff, OffsetY)

-- Set of group(s) to follow Mother.
local FollowGroupSet = SET_GROUP:New()
FollowGroupSet:AddGroupsByName(HeloSpawn:GetName())

-- Define AI Formation object.
CarrierFormation = AI_FORMATION:New(Mother, FollowGroupSet, "Helo Formation with Carrier", "Follow Carrier at given parameters.")

-- Formation parameters.
CarrierFormation:FormationCenterWing(-OffsetX, 50, math.abs(OffsetY), 50, OffsetZ, 50)

-- Start formation FSM.
CarrierFormation:__Start(2)

-- Subscribe helo group set to the event land.
FollowGroupSet:HandleEvent(EVENTS.Land)

-- Respawn helo group once it landed because it was out of fuel.
function FollowGroupSet:OnEventLand(EventData)
  local group=EventData.IniGroup --Wrapper.Group#GROUP
  if group and group:GetName():match(HeloGroupName) then
    env.info(string.format("CarrierFormation: Respawning group %s.", group:GetName()))
    group:RespawnAtCurrentAirbase(nil, Takeoff)  -- MOOSE v2.4.X
  end
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

if ReportFuel then
  --- Report fuel function of following group.
  local function ReportFuel()
    for _,myhelo in pairs(FollowGroupSet:GetSet()) do
      if myhelo and myhelo:IsAlive() then
        local fuel=myhelo:GetFuel()
        local text=string.format("%s: fuel = %.2f %%", myhelo:GetName(), fuel*100)
        MESSAGE:New(text, 5):ToAll()
      end
    end
  end
  
  -- Report fuel state of helo every 10 seconds.
  local Scheduler_Helo_Fuel = SCHEDULER:New(nil, ReportFuel, {}, 60, 60)
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------